(* $Id: pax.mli 799 2008-06-03 15:13:33Z gerd $
 * ----------------------------------------------------------------------
 * Copyright 2008 by Gerd Stolpmann.
 *
 * This file is part of godi-tools.
 *
 * godi-tools is free software; you can redistribute it and/or modify it
 * under the conditions in the file LICENSE.
 *
 * godi-tools is distributed WITHOUT ANY WARRANTY, as detailed in the
 * file LICENSE.
 *)

type in_archive
  (* An archive that is read *)

type out_archive
  (* An archive that is written to *)

type in_element
  (* An archive elements that is read *)

type in_file =
    [ `File of Godi_file.filename
    | `Stdin
    | `Obj_channel of Netchannels.in_obj_channel
    ]

type out_file =
    [ `File of Godi_file.filename
    | `Stdout
    | `Obj_channel of Netchannels.out_obj_channel
    ]

type compression_kind =
    [ `None
    | `Gzip
    | `Bzip2
    ]

type format =
    [ `Tar
    ]


val read : in_file -> compression_kind -> format -> in_archive
  (* Read from the archive. Currently, compression is not supported when
     reading from object channels
   *)

val read_element : in_archive -> in_element option
  (* Read the next element. None means that the end of the archive is
     reached.

     It is only allowed to extract the returned element as long as the
     next element is not yet retrieved (i.e. read_element is again called).
   *)

val get_metadata : in_element -> Pax_types.metadata
  (* Return the meta data of that element *)

type overwrite_mode =
    [ `Always
    | `Update
    | `Ignore
    | `Reject
    ]

val extract_element_as_file :
       ?insecure:bool ->
       ?prefer_link:bool ->
       ?prefix:Godi_file.filename ->
       ?subst:(string -> string) ->
       ?preserve_mtime:bool ->
       ?overwrite:overwrite_mode ->
       in_element -> 
         unit
  (* Extract the element. It is an error if the file name in the archive
     includes ".." as path component. Before files are created, the [subst]
     is called which may map the file name in the archive to a different
     real file name. In addition, [prefix] is prepended to the result
     (unless it is an absolute name).

     If the name is empty after applying [subst] the element is skipped.

     The file ownership attributes and the file permissions are ignored,
     except for the user bits. The mtime is not preserved by default, but
     this can be overridden with the [preserve_mtime] parameter.

     [overwrite] says which files can be overwritten:
      - `Always: all files
      - `Update: a file is only overwritten with a newer file
      - `Ignore: do silently not overwrite files
      - `Reject: stop with an error if a file had to be overwritten

     This implementation ignores elements that are neither regular files nor
     directories nor links.

     Note that directories need to be postprocessed after the extraction
     of the whole archive is done. Postprocessing actions are accumulated,
     and can be executed by calling postprocess_extraction.

     prefer_link: if true, and the element comes from a clipboard, it is
     tried to make a hard link, and only if this fails, it is fallen back
     to making a copy. 
   *)

val extract_element_to_memory : in_element -> string
  (* Returns the data part of the element as string. Only allowed for `Regular
     files
   *)

val postprocess_extraction : in_archive -> unit
  (* Executes accumulated postprocessing steps *)

     
val close_in : in_archive -> unit
  (* Close the archive. This means:
     - When it is read from stdin, or when an external command is involved,
       the remaining input is read and thrown away (to avoid EPIPE errors)
     - When a file is read, or when an object channel is read, the input
       is closed. Stdin is kept open, however.
   *)

 
val create : out_file -> compression_kind -> format -> out_archive
  (* Create an archive, and write to it. Currently, compression is not 
     supported when writing to object channels
   *)

val add_element_from_file :
      ?prefix:Godi_file.filename ->
      ?subst:(string -> string) ->
      out_archive ->
      string ->
        unit
  (* Add this file or directory to the archive. For directories, only the
     directory entry is added, but not the contents of the directory.

     Before accessing the file on the disk, the [prefix] is prepended
     to the file name.

     The [subst] function is called to get the recorded file name.

     Hard links are not recognized.

     Win32: The input file name must be a POSIX path anyway, so passing
     Win32 names is not supported here. Only the prefix may be Win32-ish.
   *)

val add_element_from_string :
      name:string ->
      mode:int ->
      mtime:int64 ->
      filetype:Pax_types.filetype ->
      out_archive ->
      string ->
        unit

val close_out : out_archive -> unit
  (* Close the archive. *)

type file_iterator

val file_iterator_of_list : 
      ?nodescent:bool -> 
      ?prefix:Godi_file.filename ->
      string list -> file_iterator
  (* Iterates over the file names in the list. Unless [nodescent] is true,
     directories are expanded. For expanding directories, the [prefix]
     argument is used to access files on the local disk.

     Win32: The input file name must be a POSIX path anyway, so passing
     Win32 names is not supported here. Only the prefix may be Win32-ish.
   *)

val file_iterator_of_channel :
      ?nodescent:bool -> 
      ?prefix:Godi_file.filename ->
      Netchannels.in_obj_channel -> file_iterator
  (* Iterates over the file names read line by line from the channel.
     The iterator does not close the channel when the iteration is done!

     Win32: The input file name must be a POSIX path anyway, so passing
     Win32 names is not supported here. Only the prefix may be Win32-ish.
   *)

val next_file : file_iterator -> string option
  (* Fetch the next file name or None if the iteration is over *)

type copier

val create_copier : unit -> copier

val copy_file : ?src_prefix:Godi_file.filename ->
                ?prefer_link:bool ->
                ?subst:(string -> string) ->
                ?preserve_mtime:bool ->
                ?overwrite:overwrite_mode ->
                copier ->
                string ->
                Godi_file.filename ->
                  unit
  (* [copy_file src dir]: Copies one file from [src] to [dir/src].
     The contents of directories are omitted.

     [src_prefix]: If passed, this prefix is prepended to [src] when
     reading the file.

     [subst]: If passed, the destination is [dir/src'] where [src' = subst src]

     Win32: The src file names must be POSIX paths anyway, so passing
     Win32 names is not supported here. Only the src_prefix and the 
     destination directory may be Win32-ish.
   *)

val postprocess_copying : copier -> unit
